# Generated by Django 5.1.5 on 2025-02-05 01:30

import django.db.models.deletion
import jestit.models.base
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('authit', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Group',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('modified', models.DateTimeField(auto_now=True, db_index=True)),
                ('name', models.CharField(max_length=200)),
                ('is_active', models.BooleanField(db_index=True, default=True)),
                ('kind', models.CharField(db_index=True, default='group', max_length=80)),
                ('permissions', models.JSONField(blank=True, default=dict)),
                ('metadata', models.JSONField(blank=True, default=dict)),
                ('parent', models.ForeignKey(default=None, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='groups', to='authit.group')),
            ],
            bases=(models.Model, jestit.models.base.JestitBase),
        ),
    ]
