# Generated by Django 5.1.5 on 2025-02-05 01:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('authit', '0002_group'),
    ]

    operations = [
        migrations.AlterField(
            model_name='user',
            name='modified',
            field=models.DateTimeField(auto_now_add=True),
        ),
    ]
