# Generated by Django 5.1.5 on 2025-02-05 01:28

import jestit.models.base
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='User',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(blank=True, null=True, verbose_name='last login')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('modified', models.DateTimeField()),
                ('last_activity', models.DateTimeField(db_index=True)),
                ('username', models.TextField(unique=True)),
                ('email', models.EmailField(max_length=254, unique=True)),
                ('phone_number', models.CharField(blank=True, default=None, max_length=32, null=True)),
                ('is_active', models.BooleanField(db_index=True, default=True)),
                ('display_name', models.CharField(blank=True, default=None, max_length=80, null=True)),
                ('auth_key', models.TextField(default=None, null=True)),
                ('onetime_code', models.TextField(default=None, null=True)),
                ('permissions', models.JSONField(blank=True, default=dict)),
                ('metadata', models.JSONField(blank=True, default=dict)),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model, jestit.models.base.JestitBase),
        ),
    ]
